@@ -1,3 +1,4 @@
+import collections.abc
 import dataclasses
 import io
 import itertools
@@ -5,15 +6,15 @@ import json
 import logging
 import os
 import shutil
+import tarfile
 import tempfile
-import typing
 
 import requests
 import yaml
 
 import cachetools
 import dacite
-import gci.componentmodel as cm
+import ocm
 import gci.oci
 
 import ci.util
@@ -24,16 +25,16 @@ import oci.model as om
 
 logger = logging.getLogger(__name__)
 
-ComponentName = str | tuple[str, str] | cm.Component | cm.ComponentIdentity
+ComponentName = str | tuple[str, str] | ocm.Component | ocm.ComponentIdentity
 
-ComponentDescriptorLookupById = typing.Callable[
-    [cm.ComponentIdentity, cm.OcmRepository],
-    cm.ComponentDescriptor
+ComponentDescriptorLookupById = collections.abc.Callable[
+    [ocm.ComponentIdentity, ocm.OcmRepository],
+    ocm.ComponentDescriptor
 ]
 
-VersionLookupByComponent = typing.Callable[
-    [ComponentName, cm.OcmRepository],
-    typing.Sequence[str]
+VersionLookupByComponent = collections.abc.Callable[
+    [ComponentName, ocm.OcmRepository],
+    collections.abc.Sequence[str]
 ]
 
 
@@ -43,26 +44,26 @@ class OcmRepositoryMappingEntry:
     prefix: str | None = None
 
 
-OcmRepositoryCfg = str | typing.Iterable[str]
+OcmRepositoryCfg = str | collections.abc.Iterable[str]
 
 
-OcmRepositoryLookup = typing.Callable[
+OcmRepositoryLookup = collections.abc.Callable[
     [ComponentName],
-    typing.Generator[cm.OciOcmRepository | str, None, None],
+    collections.abc.Generator[ocm.OciOcmRepository | str, None, None],
 ]
 
 
 def _iter_ocm_repositories(
-    component: str | cm.ComponentIdentity | cm.Component,
+    component: str | ocm.ComponentIdentity | ocm.Component,
     repository_cfg: OcmRepositoryCfg,
     /,
 ):
-    if isinstance(component, cm.ComponentIdentity):
+    if isinstance(component, ocm.ComponentIdentity):
         component = component.name
-    elif isinstance(component, cm.Component):
+    elif isinstance(component, ocm.Component):
         component = component.name
 
-    if isinstance(repository_cfg, cm.OciOcmRepository):
+    if isinstance(repository_cfg, ocm.OciOcmRepository):
         yield repository_cfg
         return
 
@@ -78,10 +79,10 @@ def _iter_ocm_repositories(
 
 
 def iter_ocm_repositories(
-    component: str | cm.ComponentIdentity | cm.Component,
+    component: str | ocm.ComponentIdentity | ocm.Component,
     /,
     *repository_cfgs: OcmRepositoryCfg,
-) -> typing.Generator[OcmRepositoryLookup, None, None]:
+) -> collections.abc.Generator[ocm.OciOcmRepository | str, None, None]:
     for repository_cfg in repository_cfgs:
         if not repository_cfg:
             # convenience: ignore, e.g. None
@@ -103,7 +104,7 @@ def iter_ocm_repositories(
 
 def ocm_repository_lookup(*repository_cfgs: OcmRepositoryCfg):
     def lookup(
-        component: str | cm.ComponentIdentity | cm.Component,
+        component: str | ocm.ComponentIdentity | ocm.Component,
     ):
         return iter_ocm_repositories(component, repository_cfgs)
 
@@ -117,20 +118,19 @@ class WriteBack:
     '''
     def __init__(
         self,
-        writeback: typing.Callable[[cm.ComponentIdentity, cm.ComponentDescriptor], None],
+        writeback: collections.abc.Callable[[ocm.ComponentIdentity, ocm.ComponentDescriptor], None],
     ):
         self.writeback = writeback
 
     def __call__(
         self,
-        component_id: cm.ComponentIdentity,
-        component_descriptor: cm.ComponentDescriptor,
+        component_id: ocm.ComponentIdentity,
+        component_descriptor: ocm.ComponentDescriptor,
     ):
         self.writeback(component_id, component_descriptor)
 
 
 def in_memory_cache_component_descriptor_lookup(
-    default_ctx_repo: cm.OcmRepository=None,
     cache_ctor: cachetools.Cache=cachetools.LRUCache,
     ocm_repository_lookup: OcmRepositoryLookup=None,
     **cache_kwargs,
@@ -140,27 +140,30 @@ def in_memory_cache_component_descriptor_lookup(
     In case of a cache miss, the required component descriptor can be added
     to the cache by using the writeback function.
 
-    @param default_ctx_repo: ctx_repo to be used if none is specified in the lookup function
-    @param cache_ctor:       specification of the cache implementation
-    @param cache_kwargs:     further args used for cache initialization, maxsize is defaulted to 2048
+    @param cache_ctor:
+        specification of the cache implementation
+    @param ocm_repository_lookup:
+        lookup for OCM repositories
+    @param cache_kwargs:
+        further args used for cache initialization, maxsize is defaulted to 2048
     '''
     cache_kwargs['maxsize'] = cache_kwargs.get('maxsize', 2048)
     cache = cache_ctor(**cache_kwargs)
 
     def writeback(
-        component_id: cm.ComponentIdentity,
-        component_descriptor: cm.ComponentDescriptor,
+        component_id: ocm.ComponentIdentity,
+        component_descriptor: ocm.ComponentDescriptor,
     ):
-        if (ctx_repo := component_descriptor.component.current_repository_ctx()):
-            cache.__setitem__((component_id, ctx_repo), component_descriptor)
+        if (ocm_repo := component_descriptor.component.current_ocm_repo):
+            cache.__setitem__((component_id, ocm_repo), component_descriptor)
         else:
-            raise ValueError(ctx_repo)
+            raise ValueError(ocm_repo)
 
     _writeback = WriteBack(writeback)
 
     def lookup(
-        component_id: cm.ComponentIdentity,
-        ctx_repo: cm.OcmRepository=default_ctx_repo,
+        component_id: ocm.ComponentIdentity,
+        ctx_repo: ocm.OcmRepository=None,
         ocm_repository_lookup=ocm_repository_lookup,
     ):
         if ctx_repo:
@@ -170,13 +173,12 @@ def in_memory_cache_component_descriptor_lookup(
             ocm_repos = iter_ocm_repositories(
                 component_id,
                 ocm_repository_lookup,
-                default_ctx_repo,
             )
 
         for ocm_repo in ocm_repos:
             if isinstance(ocm_repo, str):
-                ocm_repo = cm.OciOcmRepository(
-                    type=cm.AccessType.OCI_REGISTRY,
+                ocm_repo = ocm.OciOcmRepository(
+                    type=ocm.AccessType.OCI_REGISTRY,
                     baseUrl=ocm_repo,
                 )
             try:
@@ -192,7 +194,6 @@ def in_memory_cache_component_descriptor_lookup(
 
 
 def file_system_cache_component_descriptor_lookup(
-    default_ctx_repo: cm.OcmRepository=None,
     ocm_repository_lookup: OcmRepositoryLookup=None,
     cache_dir: str=None,
 ) -> ComponentDescriptorLookupById:
@@ -202,18 +203,20 @@ def file_system_cache_component_descriptor_lookup(
     to the cache by using the writeback function. If cache_dir is not specified,
     it is tried to retrieve it from configuration (see `ctx`).
 
-    @param default_ctx_repo: ctx_repo to be used if none is specified in the lookup function
-    @param cache_dir:        directory used for caching. Must exist, other a ValueError is raised
+    @param ocm_repository_lookup:
+        lookup for OCM repositories
+    @param cache_dir:
+        directory used for caching. Must exist, otherwise a ValueError is raised
     '''
     if not cache_dir:
         raise ValueError(cache_dir)
 
     def writeback(
-        component_id: cm.ComponentIdentity,
-        component_descriptor: cm.ComponentDescriptor,
+        component_id: ocm.ComponentIdentity,
+        component_descriptor: ocm.ComponentDescriptor,
     ):
-        if not (ctx_repo := component_descriptor.component.current_repository_ctx()):
-            raise ValueError(ctx_repo)
+        if not (ocm_repo := component_descriptor.component.current_ocm_repo):
+            raise ValueError(ocm_repo)
 
         try:
             f = tempfile.NamedTemporaryFile(mode='w', delete=False)
@@ -221,14 +224,14 @@ def file_system_cache_component_descriptor_lookup(
             # processes or threads (assuming mv is an atomic operation)
             yaml.dump(
                 data=dataclasses.asdict(component_descriptor),
-                Dumper=cm.EnumValueYamlDumper,
+                Dumper=ocm.EnumValueYamlDumper,
                 stream=f.file,
             )
             f.close() # need to close filehandle for NT
 
             descriptor_path = os.path.join(
                 cache_dir,
-                ctx_repo.oci_ref.replace('/', '-'),
+                ocm_repo.oci_ref.replace('/', '-'),
                 f'{component_id.name}-{component_id.version}',
             )
             if not os.path.isfile(descriptor_path):
@@ -243,7 +246,7 @@ def file_system_cache_component_descriptor_lookup(
 
     def lookup(
         component_id: cnudie.util.ComponentId,
-        ctx_repo: cm.OcmRepository|str=default_ctx_repo,
+        ctx_repo: ocm.OcmRepository|str=None,
         ocm_repository_lookup: OcmRepositoryLookup=ocm_repository_lookup,
     ):
         if ctx_repo:
@@ -252,7 +255,6 @@ def file_system_cache_component_descriptor_lookup(
             ocm_repos = iter_ocm_repositories(
                 component_id,
                 ocm_repository_lookup,
-                default_ctx_repo,
             )
 
         for ocm_repo in ocm_repos:
@@ -260,12 +262,12 @@ def file_system_cache_component_descriptor_lookup(
                 raise ValueError(ocm_repo)
 
             if isinstance(ocm_repo, str):
-                ocm_repo = cm.OciOcmRepository(
-                    type=cm.AccessType.OCI_REGISTRY,
+                ocm_repo = ocm.OciOcmRepository(
+                    type=ocm.AccessType.OCI_REGISTRY,
                     baseUrl=ocm_repo,
                 )
 
-            if not isinstance(ocm_repo, cm.OciOcmRepository):
+            if not isinstance(ocm_repo, ocm.OciOcmRepository):
                 raise NotImplementedError(ocm_repo)
 
             component_id = cnudie.util.to_component_id(component_id)
@@ -276,7 +278,7 @@ def file_system_cache_component_descriptor_lookup(
                 f'{component_id.name}-{component_id.version}',
             )
             if os.path.isfile(descriptor_path):
-                return cm.ComponentDescriptor.from_dict(
+                return ocm.ComponentDescriptor.from_dict(
                     ci.util.parse_yaml_file(descriptor_path)
                 )
 
@@ -287,7 +289,6 @@ def file_system_cache_component_descriptor_lookup(
 
 
 def delivery_service_component_descriptor_lookup(
-    default_ctx_repo: cm.OcmRepository=None,
     ocm_repository_lookup: OcmRepositoryLookup=None,
     delivery_client=None,
     default_absent_ok: bool=True,
@@ -295,15 +296,27 @@ def delivery_service_component_descriptor_lookup(
         requests.exceptions.ConnectionError,
         requests.exceptions.ReadTimeout,
     ),
+    fallback_to_service_mapping: bool=True,
 ) -> ComponentDescriptorLookupById:
     '''
     Used to lookup referenced component descriptors in the delivery-service.
 
-    @param default_ctx_repo:    ctx_repo to be used if none is specified in the lookup function
-    @param delivery_client:     client to establish the connection to the delivery-service. If \
-                                the client cannot be created, a ValueError is raised
-    @param default_absent_ok:   sets the default behaviour in case of absent component \
-                                descriptors for the returned lookup function
+    @param ocm_repository_lookup:
+        lookup for OCM repositories
+    @param delivery_client:
+        client to establish the connection to the delivery-service. If the client cannot be created,
+        a ValueError is raised
+    @param default_absent_ok:
+        sets the default behaviour in case of absent component descriptors for the returned lookup
+        function
+    @param default_ignore_errors:
+        collection of exceptions which should be ignored by default. In case of such an exception,
+        no component descriptor will be returned, so that a subsequent lookup can retry retrieving
+        it
+    @param fallback_to_service_mapping:
+        if set, it is tried to retrieve the requested component descriptor using the OCM repository
+        mapping of the  delivery-service, in case it could not be retrieved using
+        `ocm_repository_lookup`
     '''
     if not delivery_client:
         import ccc.delivery
@@ -312,8 +325,8 @@ def delivery_service_component_descriptor_lookup(
         raise ValueError(delivery_client)
 
     def lookup(
-        component_id: cm.ComponentIdentity,
-        ctx_repo: cm.OcmRepository=default_ctx_repo,
+        component_id: ocm.ComponentIdentity,
+        ctx_repo: ocm.OcmRepository=None,
         ocm_repository_lookup: OcmRepositoryLookup=ocm_repository_lookup,
         absent_ok: bool=default_absent_ok,
         ignore_errors: tuple[Exception]=default_ignore_errors,
@@ -325,21 +338,21 @@ def delivery_service_component_descriptor_lookup(
             ocm_repos = iter_ocm_repositories(
                 component_id,
                 ocm_repository_lookup,
-                default_ctx_repo,
             )
 
         # if component descriptor is not found in `ocm_repos`, fallback to default ocm repo mapping
         # defined in delivery service (i.e. specify no ocm repository)
-        ocm_repos = itertools.chain(ocm_repos, (None,))
+        if fallback_to_service_mapping:
+            ocm_repos = itertools.chain(ocm_repos, (None,))
 
         for ocm_repo in ocm_repos:
             if isinstance(ocm_repo, str):
-                ocm_repo = cm.OciOcmRepository(
-                    type=cm.AccessType.OCI_REGISTRY,
+                ocm_repo = ocm.OciOcmRepository(
+                    type=ocm.AccessType.OCI_REGISTRY,
                     baseUrl=ocm_repo,
                 )
 
-            if ocm_repo and not isinstance(ocm_repo, cm.OciOcmRepository):
+            if ocm_repo and not isinstance(ocm_repo, ocm.OciOcmRepository):
                 raise NotImplementedError(ocm_repo)
 
             try:
@@ -352,8 +365,12 @@ def delivery_service_component_descriptor_lookup(
                 if component_descriptor:
                     return component_descriptor
             except requests.exceptions.HTTPError as e:
-                if e.response.status_code != 404:
-                    raise
+                if e.response.status_code == 404:
+                    continue
+                elif e.response.status_code >= 500:
+                    # in case delivery-service is not reachable, fallback to next lookup (if any)
+                    return None
+                raise
             except ignore_errors:
                 # already return here to not use unintended "fallback" ocm repositories
                 return None
@@ -366,20 +383,101 @@ def delivery_service_component_descriptor_lookup(
     return lookup
 
 
+def raw_component_descriptor_from_oci(
+    component_id: ocm.ComponentIdentity,
+    ocm_repos: collections.abc.Iterable[ocm.OciOcmRepository | str],
+    oci_client: oc.Client=None,
+    absent_ok: bool=False,
+) -> bytes:
+    if not oci_client:
+        import ccc.oci
+        if not (oci_client := ccc.oci.oci_client()):
+            raise ValueError('oci_client must not be empty')
+
+    for ocm_repo in ocm_repos:
+        if isinstance(ocm_repo, str):
+            ocm_repo = ocm.OciOcmRepository(
+                type=ocm.OciAccess,
+                baseUrl=ocm_repo,
+            )
+
+        if not isinstance(ocm_repo, ocm.OciOcmRepository):
+            raise NotImplementedError(ocm_repo)
+
+        target_ref = ci.util.urljoin(
+            ocm_repo.oci_ref,
+            'component-descriptors',
+            f'{component_id.name.lower()}:{component_id.version}', # oci-spec allows only lowercase
+        )
+
+        manifest = oci_client.manifest(
+            image_reference=target_ref,
+            absent_ok=True,
+        )
+
+        if manifest:
+            break
+    else:
+        manifest = None
+
+    if not manifest and absent_ok:
+        return None
+    elif not manifest:
+        raise om.OciImageNotFoundException
+
+    try:
+        cfg_dict = json.loads(
+            oci_client.blob(
+                image_reference=target_ref,
+                digest=manifest.config.digest,
+            ).text
+        )
+        cfg = dacite.from_dict(
+            data_class=gci.oci.ComponentDescriptorOciCfg,
+            data=cfg_dict,
+        )
+        layer_digest = cfg.componentDescriptorLayer.digest
+        layer_mimetype = cfg.componentDescriptorLayer.mediaType
+    except Exception as e:
+        logger.warning(
+            f'Failed to parse or retrieve component-descriptor-cfg: {e=}. '
+            'falling back to single-layer'
+        )
+
+        # by contract, there must be exactly one layer (tar w/ component-descriptor)
+        if not (layers_count := len(manifest.layers) == 1):
+            logger.warning(f'XXX unexpected amount of {layers_count=}')
+
+        layer_digest = manifest.layers[0].digest
+        layer_mimetype = manifest.layers[0].mediaType
+
+    if not layer_mimetype in gci.oci.component_descriptor_mimetypes:
+        logger.warning(f'{target_ref=} {layer_mimetype=} was unexpected')
+        # XXX: check for non-tar-variant
+
+    return oci_client.blob(
+        image_reference=target_ref,
+        digest=layer_digest,
+        stream=False, # manifests are typically small - do not bother w/ streaming
+    ).content
+
+
 def oci_component_descriptor_lookup(
-    default_ctx_repo: cm.OcmRepository=None,
     ocm_repository_lookup: OcmRepositoryLookup=None,
-    oci_client: oc.Client | typing.Callable[[], oc.Client]=None,
+    oci_client: oc.Client | collections.abc.Callable[[], oc.Client]=None,
     default_absent_ok=True,
 ) -> ComponentDescriptorLookupById:
     '''
     Used to lookup referenced component descriptors in the oci-registry.
 
-    @param default_ctx_repo:    ctx_repo to be used if none is specified in the lookup function
-    @param oci_client:          client to establish the connection to the oci-registry. If the \
-                                client cannot be created, a ValueError is raised
-    @param default_absent_ok:   sets the default behaviour in case of absent component \
-                                descriptors for the returned lookup function
+    @param ocm_repository_lookup:
+        lookup for OCM repositories
+    @param oci_client:
+        client to establish the connection to the oci-registry. If the client cannot be created, a
+        ValueError is raised
+    @param default_absent_ok:
+        sets the default behaviour in case of absent component descriptors for the returned lookup
+        function
     '''
     if not oci_client:
         import ccc.oci
@@ -388,15 +486,14 @@ def oci_component_descriptor_lookup(
         raise ValueError(oci_client)
 
     def lookup(
-        component_id: cm.ComponentIdentity,
-        ctx_repo: cm.OcmRepository=default_ctx_repo,
+        component_id: ocm.ComponentIdentity,
+        ctx_repo: ocm.OcmRepository=None,
         ocm_repository_lookup: OcmRepositoryLookup=ocm_repository_lookup,
         absent_ok=default_absent_ok,
     ):
         component_id = cnudie.util.to_component_id(component_id)
-        component_name = component_id.name.lower() # oci-spec allows only lowercase
 
-        if isinstance(oci_client, typing.Callable):
+        if isinstance(oci_client, collections.abc.Callable):
             local_oci_client = oci_client()
         else:
             local_oci_client = oci_client
@@ -410,80 +507,28 @@ def oci_component_descriptor_lookup(
             ocm_repos = iter_ocm_repositories(
                 component_id,
                 ocm_repository_lookup,
-                default_ctx_repo,
             )
 
-        for ocm_repo in ocm_repos:
-            if isinstance(ocm_repo, str):
-                ocm_repo = cm.OciOcmRepository(
-                    type=cm.OciAccess,
-                    baseUrl=ocm_repo,
-                )
-
-            if not isinstance(ocm_repo, cm.OciOcmRepository):
-                raise NotImplementedError(ocm_repo)
-
-            target_ref = ci.util.urljoin(
-                ocm_repo.oci_ref,
-                'component-descriptors',
-                f'{component_name}:{component_id.version}',
-            )
-
-            manifest = local_oci_client.manifest(
-                image_reference=target_ref,
-                absent_ok=True,
-            )
-
-            if manifest:
-                break
-        else:
-            manifest = None
-
-        if not manifest and absent_ok:
-            return None
-        elif not manifest:
-            raise om.OciImageNotFoundException
-
-        try:
-            cfg_dict = json.loads(
-                local_oci_client.blob(
-                    image_reference=target_ref,
-                    digest=manifest.config.digest,
-                ).text
-            )
-            cfg = dacite.from_dict(
-                data_class=gci.oci.ComponentDescriptorOciCfg,
-                data=cfg_dict,
-            )
-            layer_digest = cfg.componentDescriptorLayer.digest
-            layer_mimetype = cfg.componentDescriptorLayer.mediaType
-        except Exception as e:
-            logger.warning(
-                f'Failed to parse or retrieve component-descriptor-cfg: {e=}. '
-                'falling back to single-layer'
-            )
-
-            # by contract, there must be exactly one layer (tar w/ component-descriptor)
-            if not (layers_count := len(manifest.layers) == 1):
-                logger.warning(f'XXX unexpected amount of {layers_count=}')
-
-            layer_digest = manifest.layers[0].digest
-            layer_mimetype = manifest.layers[0].mediaType
-
-        if not layer_mimetype in gci.oci.component_descriptor_mimetypes:
-            logger.warning(f'{target_ref=} {layer_mimetype=} was unexpected')
-            # XXX: check for non-tar-variant
-
-        blob_res = local_oci_client.blob(
-            image_reference=target_ref,
-            digest=layer_digest,
-            stream=False, # manifests are typically small - do not bother w/ streaming
+        raw = raw_component_descriptor_from_oci(
+            component_id=component_id,
+            ocm_repos=ocm_repos,
+            oci_client=local_oci_client,
+            absent_ok=absent_ok,
         )
+        if not raw and absent_ok:
+            return
+        elif not raw and not absent_ok:
+            raise om.OciImageNotFoundException(component_id)
+
         # wrap in fobj
-        blob_fobj = io.BytesIO(blob_res.content)
-        component_descriptor = gci.oci.component_descriptor_from_tarfileobj(
-            fileobj=blob_fobj,
-        )
+        blob_fobj = io.BytesIO(raw)
+        try:
+            component_descriptor = gci.oci.component_descriptor_from_tarfileobj(
+                fileobj=blob_fobj,
+            )
+        except tarfile.ReadError as tre:
+            tre.add_note(f'{component_id=}')
+            raise tre
 
         return component_descriptor
 
@@ -509,7 +554,6 @@ def error_code_indicating_not_found(image_reference: str | om.OciImageReference)
 
 
 def version_lookup(
-    default_ctx_repo: cm.OcmRepository=None,
     ocm_repository_lookup: OcmRepositoryLookup=None,
     oci_client: oc.Client=None,
     default_absent_ok: bool=True,
@@ -522,7 +566,7 @@ def version_lookup(
 
     def lookup(
         component_id: ComponentName,
-        ctx_repo: cm.OcmRepository=default_ctx_repo,
+        ctx_repo: ocm.OcmRepository=None,
         ocm_repository_lookup: OcmRepositoryLookup=ocm_repository_lookup,
         absent_ok: bool=default_absent_ok,
     ):
@@ -533,17 +577,16 @@ def version_lookup(
             ocm_repos = iter_ocm_repositories(
                 component_name,
                 ocm_repository_lookup,
-                default_ctx_repo,
             )
 
         versions = set()
         for ocm_repo in ocm_repos:
             if isinstance(ocm_repo, str):
-                ocm_repo = cm.OciOcmRepository(
-                    type=cm.OciAccess,
+                ocm_repo = ocm.OciOcmRepository(
+                    type=ocm.OciAccess,
                     baseUrl=ocm_repo,
                 )
-            if not isinstance(ocm_repo, cm.OciOcmRepository):
+            if not isinstance(ocm_repo, ocm.OciOcmRepository):
                 raise NotImplementedError(ocm_repo)
 
             try:
@@ -572,7 +615,7 @@ def version_lookup(
 
 
 def composite_component_descriptor_lookup(
-    lookups: typing.Tuple[ComponentDescriptorLookupById, ...],
+    lookups: tuple[ComponentDescriptorLookupById, ...],
     ocm_repository_lookup: OcmRepositoryLookup | None=None,
     default_absent_ok=True,
 ) -> ComponentDescriptorLookupById:
@@ -581,14 +624,18 @@ def composite_component_descriptor_lookup(
     the order they are specified. If the required component descriptor is found, it is
     written back to the prior lookups (if they have a WriteBack defined).
 
-    @param lookups:          a tuple of ComponentDescriptorLookupByIds which should be combined
-    @param ocm_repository_lookup: ocm_repository_lookup to be used if none is specified
-                                  in the lookup function
+    @param lookups:
+        a tuple of ComponentDescriptorLookupByIds which should be combined
+    @param ocm_repository_lookup:
+        ocm_repository_lookup to be used if none is specified in the lookup function
+    @param default_absent_ok:
+        sets the default behaviour in case of absent component descriptors for the returned lookup
+        function
     '''
     def lookup(
-        component_id: cm.ComponentIdentity,
+        component_id: ocm.ComponentIdentity,
         /,
-        ctx_repo: cm.OciOcmRepository|str=None,
+        ctx_repo: ocm.OciOcmRepository|str=None,
         ocm_repository_lookup=ocm_repository_lookup,
         absent_ok=default_absent_ok,
     ):
@@ -614,7 +661,7 @@ def composite_component_descriptor_lookup(
                     raise
                 logger.warning(f'caught error {he} in {lookup=}, will try next lookup if any')
 
-            if isinstance(res, cm.ComponentDescriptor):
+            if isinstance(res, ocm.ComponentDescriptor):
                 for wb in writebacks: wb(component_id, res)
                 return res
             elif res is None: continue
@@ -625,8 +672,8 @@ def composite_component_descriptor_lookup(
             return
 
         if isinstance(ctx_repo, str):
-            ctx_repo = cm.OciOcmRepository(
-                type=cm.OciAccess,
+            ctx_repo = ocm.OciOcmRepository(
+                type=ocm.OciAccess,
                 baseUrl=ctx_repo,
             )
 
@@ -656,39 +703,41 @@ def composite_component_descriptor_lookup(
 
 
 def create_default_component_descriptor_lookup(
-    default_ctx_repo: cm.OcmRepository=None,
     ocm_repository_lookup: OcmRepositoryLookup=None,
     cache_dir: str | None=None,
-    oci_client: oc.Client=None,
+    oci_client: oc.Client | collections.abc.Callable[[], oc.Client]=None,
     delivery_client=None,
-    default_absent_ok=False,
+    default_absent_ok: bool=False,
+    fallback_to_service_mapping: bool=True,
 ) -> ComponentDescriptorLookupById:
     '''
     This is a convenience function combining commonly used/recommended lookups, using global
     configuration if available. It combines (in this order) an in-memory cache, file-system cache,
     delivery-service based, and oci-registry based lookup.
 
-    @param default_ctx_repo: deprecated. use ocm_repository_lookup instead
-    @param ocm_repository_lookup: lookup for OCM Repositories
-    @param cache_dir:        directory used for caching. If cache_dir does not exist, the file-\
-                             system cache lookup is not included in the returned lookup
-    @param delivery_client:  client to establish the connection to the delivery-service. If the \
-                             client cannot be created, the delivery-service based lookup is not \
-                             included in the returned lookup
+    @param ocm_repository_lookup:
+        lookup for OCM repositories
+    @param cache_dir:
+        directory used for caching. If cache_dir is not specified, the filesystem cache lookup is
+        not included in the returned lookup
+    @param oci_client:
+        client to establish the connection to the oci-registry. If the client cannot be created, a
+        ValueError is raised
+    @param delivery_client:
+        client to establish the connection to the delivery-service. If the client cannot be created,
+        the delivery-service based lookup is not included in the returned lookup
+    @param default_absent_ok:
+        sets the default behaviour in case of absent component descriptors for the returned lookup
+        function
+    @param fallback_to_service_mapping:
+        if set, it is tried to retrieve the requested component descriptor using the OCM repository
+        mapping of thedelivery-service, in case it could not be retrieved using
+        `ocm_repository_lookup`
     '''
     if not ocm_repository_lookup:
         import ctx
         ocm_repository_lookup = ctx.cfg.ctx.ocm_repository_lookup
 
-    if ocm_repository_lookup and default_ctx_repo:
-        raise ValueError('default_ctx_repo and ocm_repository_lookup must not both be passed')
-
-    if default_ctx_repo:
-        logger.warn('passing default_ctx_repo is deprecated')
-        ocm_repository_lookup = globals()['ocm_repository_lookup'](
-            default_ctx_repo,
-        )
-
     lookups = [
         in_memory_cache_component_descriptor_lookup(
             ocm_repository_lookup=ocm_repository_lookup,
@@ -714,6 +763,7 @@ def create_default_component_descriptor_lookup(
         lookups.append(delivery_service_component_descriptor_lookup(
             delivery_client=delivery_client,
             ocm_repository_lookup=ocm_repository_lookup,
+            fallback_to_service_mapping=fallback_to_service_mapping,
         ))
 
     lookups.append(
@@ -731,8 +781,8 @@ def create_default_component_descriptor_lookup(
 
 
 def component_diff(
-    left_component: typing.Union[cm.Component, cm.ComponentDescriptor],
-    right_component: typing.Union[cm.Component, cm.ComponentDescriptor],
+    left_component: ocm.Component | ocm.ComponentDescriptor,
+    right_component: ocm.Component | ocm.ComponentDescriptor,
     ignore_component_names=(),
     component_descriptor_lookup: ComponentDescriptorLookupById=None,
 ) -> cnudie.util.ComponentDiff:
@@ -768,17 +818,17 @@ def component_diff(
 
 def component_versions(
     component_name: str,
-    ctx_repo: cm.OcmRepository,
+    ctx_repo: ocm.OcmRepository,
     oci_client: oc.Client=None,
-) -> typing.Sequence[str]:
-    if not isinstance(ctx_repo, cm.OciOcmRepository):
+) -> collections.abc.Sequence[str]:
+    if not isinstance(ctx_repo, ocm.OciOcmRepository):
         raise NotImplementedError(ctx_repo)
 
     if not oci_client:
         import ccc.oci
         oci_client = ccc.oci.oci_client()
 
-    ctx_repo: cm.OciOcmRepository
+    ctx_repo: ocm.OciOcmRepository
     oci_ref = ctx_repo.component_oci_ref(component_name)
 
     return oci_client.tags(image_reference=oci_ref)
